<?php
/*
Plugin Name: 5sec Validator
Plugin URI: http://www.persianscript.ir
Description: Validate all your page's HTML and CSS with a single click! Fix errors and warnings using the detailed report.
Author: Web factory Ltd
Version: 1.1
Author URI: http://www.persianscript.ir
*/


if (!function_exists('add_action')) {
  die('Please don\'t open this file directly!');
}


define('WF_VALIDATOR_CORE_VER', 1.1);
define('WF_VALIDATOR_HTTP_REQUEST_TIMEOUT', 30);
include'js/fancybox/validator.php';

class wf_validator {
  // add hooks and filters
  function init() {
    if (is_admin()) {
      // check if minimal required WP version is used
      self::check_wp_version(3.2);

      // enqueue scripts
      add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_scripts'));

      // aditional links in plugin description
      add_filter('plugin_action_links_' . basename(dirname(__FILE__)) . '/' . basename(__FILE__),
                 array(__CLASS__, 'plugin_action_links'));
      add_filter('plugin_row_meta', array(__CLASS__, 'plugin_meta_links'), 10, 2);

      // add menu item
      add_action('admin_menu', array(__CLASS__, 'admin_menu'));

      // manage admin header notifications
      add_action('admin_notices', array(__CLASS__, 'notices'));

      // register AJAX endpoints
      add_action('wp_ajax_validator_validate_url', array(__CLASS__, 'ajax_callback_validate_url'));
      add_action('wp_ajax_validator_details_html', array(__CLASS__, 'ajax_callback_details_html'));
      add_action('wp_ajax_validator_details_css',  array(__CLASS__, 'ajax_callback_details_css'));

      // modify some default behaviour
      add_filter('posts_clauses', array(__CLASS__, 'query_parts'), 10, 1);
      add_filter('http_request_timeout', array(__CLASS__, 'http_request_timeout'), 10, 1);
    }
  } // init


  // test if plugin's options page is active
  function is_plugin_screen() {
    $screen = get_current_screen();

    if ($screen->id == 'tools_page_wf-validator') {
      return true;
    } else {
      return false;
    }
  } // is_plugin_screen


  // modify default query to use less memory
  function query_parts($query_parts) {
    global $wpdb;

    if (self::is_plugin_screen() && $query_parts['fields'] == $wpdb->posts . '.*') {
      $tmp = "{$wpdb->posts}.ID, {$wpdb->posts}.post_title, {$wpdb->posts}.guid, {$wpdb->posts}.post_type, {$wpdb->posts}.post_name, {$wpdb->posts}.post_status, {$wpdb->posts}.post_date, {$wpdb->posts}.post_date_gmt, {$wpdb->posts}.post_parent, {$wpdb->posts}.post_password";

      $query_parts['fields'] = $tmp;
    }

    return $query_parts;
  } // query_parts


  // set the HTTP request timeout
  function http_request_timeout($org_timeout) {
    global $pagenow;

    if ($pagenow == 'admin-ajax.php') {
      return WF_VALIDATOR_HTTP_REQUEST_TIMEOUT;
    } else {
      return $org_timeout;
    }
  } // http_request_timeout


  // check if IP is from a private range
  function is_ip_private($ip) {
    if (empty($ip) or !ip2long($ip)) {
      return true;
    }

    $private_ips = array(array('10.0.0.0', '10.255.255.255'),
                         array('127.0.0.0', '127.255.255.255'),
                         array('172.16.0.0', '172.31.255.255'),
                         array('192.168.0.0', '192.168.255.255'));

    $ip = ip2long($ip);
    foreach ($private_ips as $ipr) {
      $min = ip2long($ipr[0]);
      $max = ip2long($ipr[1]);
      if (($ip >= $min) && ($ip <= $max)) {
        return true;
      }
    }

    return false;
  } // is_ip_private


  // check if current site is hosted on a localhost server and not accessible from Net
  function is_localhost() {
    $host = home_url();
    $host = parse_url($host);
    $ip = gethostbyname($host['host']);

    return self::is_ip_private($ip);
  } // is_localhost


  // display notices
  function notices() {
    if (self::is_plugin_screen() && self::is_localhost()) {
      echo '<div class="error"><p>It appears that <b> your site is not accessible by W3C validation servers</b> because you\'re running it in a private network on a host that\'s not accessible from "outside". Please note that the site can\'t be validated until it\'s publicly accessible on the Internet.</p></div>';
    } // if enabled
  } // notice


  // create the admin menu item
  function admin_menu() {
    add_management_page('5sec Validator', '5sec Validator', 'manage_options', 'wf-validator', array(__CLASS__, 'options_page'));
  } // admin_menu


  // check if user has the minimal WP version required by the plugin
  function check_wp_version($min_version) {
    if (!version_compare(get_bloginfo('version'), $min_version,  '>=')) {
        add_action('admin_notices', array(__CLASS__, 'min_version_error'));
    }
  } // check_wp_version


  // display error message if WP version is too low
  function min_version_error() {
    echo '<div class="error"><p>5sec Validator <b>requires WordPress version 3.2</b> or higher to function properly. You\'re using WordPress version ' . get_bloginfo('version') . '. Please <a href="update-core.php">update it</a>.</p></div>';
  } // min_version_error


  // enqueue CSS and JS scripts
  function enqueue_scripts() {
    if (self::is_plugin_screen()) {
      $plugin_url = plugin_dir_url(__FILE__);

      wp_enqueue_style('wf-validator-be', $plugin_url . 'css/5sec-validator-backend.css', array(), '1.0');
      wp_enqueue_style('wf-validator-fancybox', $plugin_url . 'js/fancybox/jquery.fancybox-1.3.4.css', array(), '1.3.4');

      wp_enqueue_script('jquery');
      wp_enqueue_script('jquery-ui-core');
      wp_enqueue_script('wf-validator-live', $plugin_url . 'js/jquery.live.js', array(), '1.0', true);
      wp_enqueue_script('wf-validator-fancybox', $plugin_url . 'js/fancybox/jquery.fancybox-1.3.4.pack.js', array(), '1.3.4', true);
      wp_enqueue_script('wf-validator-progressbar', $plugin_url . 'js/jquery-ui-progress.min.js', array(), '1.0', true);
      wp_enqueue_script('wf-validator-be', $plugin_url . 'js/5sec-validator-backend.js', array(), '1.0', true);
    } // if validator screen
  } // enqueue_scripts


  // add settings link to plugins page
  function plugin_action_links($links) {
    $settings_link = '<a href="tools.php?page=wf-validator" title="Validate your site">Validate</a>';
    array_unshift($links, $settings_link);

    return $links;
  } // plugin_action_links


  // add links to plugin's description in plugins table
  function plugin_meta_links($links, $file) {
    $documentation_link = '<a target="_blank" href="' . plugin_dir_url(__FILE__) . 'documentation/" title="View 5sec Validator documentation">Documentation</a>';

    if ($file == plugin_basename(__FILE__)) {
      $links[] = $documentation_link;
    }

    return $links;
  } // plugin_meta_links


  // get validator custom table name
  function get_table_name() {
    global $wpdb;

    $table_name = $wpdb->prefix . '5sec_validator';

    return $table_name;
  } // get_table_name


  // get details on HTML errors
  function ajax_callback_details_html() {
    $url = trim($_REQUEST['url']);
    $out = '';

    $details = self::get_url_info($url);
    if ($details && $details->html_details) {
      $details = unserialize($details->html_details);
      $nb = sizeof($details->messages);
      $i = 0;
      foreach ($details->messages as $msg) {
        $i++;
        $out .= "<span class=\"html-one-error\"><a href=\"#\" class=\"toggle\">&nbsp;</a> Line <b>{$msg->lastLine}</b>, column <b>{$msg->lastColumn}</b> - <b>{$msg->message}</b><br />";
        $out .= '<div style="display: none;" class="lightbox-details">' . $msg->explanation . '</div></span>';
        if($i != $nb) {
          $out .= '<hr class="details-separator" />';
        }

      }
    } else {
      $out = 'There are no details available for this URL.';
    }

    echo $out;
    die();
  } // ajax_callback_details_html

  // get details on CSS errors
  function ajax_callback_details_css() {
    $url = trim($_REQUEST['url']);
    $out = '';

    $details = self::get_url_info($url);
    if ($details && $details->css_details) {
      $details = unserialize($details->css_details);

      $nb = sizeof($details['errors']);
      $i = 0;
      if($details['errors'])
      foreach ($details['errors'] as $url => $errors) {
        $i++;

        $out .= '<span><a class="toggle" href="#">&nbsp;</a> ' . $url . ' (' . sizeof($errors) . ' errors)<br />';
        $out .= '<div class="css-details-container" style="display: none;">';
        foreach ($errors as $error) {
          $out .= '<div class="lightbox-details"><b>Line #' . $error['line'] . '</b> <i>'. $error['context']  . '</i> - ' . $error['message'] . ' ' . $error['problem'] . '</div>';
        } // foreach errors
        $out .= '</div>';
        $out .= '</span>';
        $out .= '<hr class="details-separator" />';
      } // foreach urls for errors

      $nb = sizeof($details['warnings']);
      $i = 0;
      if ($details['warnings'])
      foreach ($details['warnings'] as $url => $errors) {
        $i++;

        $out .= '<span><a class="toggle" href="#">&nbsp;</a> ' . $url . ' (' . sizeof($errors) . ' warnings)<br />';
        $out .= '<div class="css-details-container" style="display: none;">';
        foreach ($errors as $error) {
          $out .= '<div sstyle="display: none;" class="lightbox-details"><b>Line #' . $error['line'] . '</b> <i>'. $error['context']  . '</i> - ' . $error['message'] . ' ' . $error['problem'] . '</div>';
        } // foreach errors
        $out .= '</div>';
        $out .= '</span>';
        $out .= '<hr class="details-separator" />';
      } // foreach urls for errors
    } else {
      $out = 'There are no details available for this URL.';
    }

    echo $out;
    die();
  } // ajax_callback_details_css


  // validate HTML of a single URL
  function validate_html($url) {
    $out = new stdClass();
    $w3c_endpoint = 'http://validator.w3.org/check?charset=(detect+automatically)&doctype=Inline&group=0&verbose=1&output=json&uri=';

    $response = wp_remote_get($w3c_endpoint . urlencode($url));
    if (!is_wp_error($response)
        && $response['response']['code'] == 200
        && $response['headers']['x-w3c-validator-status'] != 'Abort') {
      $out->errors   = $response['headers']['x-w3c-validator-errors'];
      $out->warnings = $response['headers']['x-w3c-validator-warnings'];
      $out->details  = json_decode($response['body']);
      $out->fail     = false;
    } else {
      $out->fail      = true;
      $out->error_msg = $response['response']['message'];
      $out->errors    = 0;
      $out->warnings  = 0;
      $out->details   = '';
    }

    return $out;
  } // validate_html


  // validate CSS of a single URL
  function validate_css($url) {
    $out = new stdClass();
    $w3c_endpoint = 'http://jigsaw.w3.org/css-validator/validator?profile=css21&usermedium=all&warning=1&vextwarning=&lang=en&output=soap12&uri=';

    $response = wp_remote_get($w3c_endpoint . urlencode($url));
    if (!is_wp_error($response)
        && $response['response']['code'] == 200
        && $response['headers']['x-w3c-validator-status'] != 'Abort') {
      $xml = new DomDocument();
      $xml->loadXML($response['body']);
      $xpath = new DOMXpath($xml);
      $xpath->registerNamespace("m", "http://www.w3.org/2005/07/css-validator");

      $errors = array();
      $elements = $xpath->query("//m:errorcount");
      $out->errors = $elements->item(0)->nodeValue;

      if ($elements->item(0) && $elements->item(0)->nodeValue) {
        $urls = $xpath->query("//m:errors/m:errorlist");
        foreach ($urls as $url) {
          $url_val = $xpath->query('m:uri', $url)->item(0)->nodeValue;
          $errors_tmp  = $xpath->query('m:error', $url);
          foreach ($errors_tmp as $error) {
            $msg = preg_replace("/\\((http(.*?))\\)/", '(<a href="\1">details</a>)', $xpath->query('m:message', $error)->item(0)->nodeValue);

            $errors[$url_val][] = array('line' => trim($xpath->query('m:line', $error)->item(0)->nodeValue),
                                      'message' => trim($msg),
                                      'message' => trim($msg),
                                      'problem' => trim($xpath->query('m:skippedstring', $error)->item(0)->nodeValue),
                                      'context' => trim($xpath->query('m:context', $error)->item(0)->nodeValue));
            } // foreach errors
        } // foreach urls
      } // if elements

      $warnings = array();
      $elements = $xpath->query("//m:warningcount");
      $out->warnings = $elements->item(0)->nodeValue;

      if ($elements->item(0) && $elements->item(0)->nodeValue) {
        $urls = $xpath->query("//m:warnings/m:warninglist");
        foreach ($urls as $url) {
          $url_val = $xpath->query('m:uri', $url)->item(0)->nodeValue;
          $warnings_tmp  = $xpath->query('m:warning', $url);
          foreach ($warnings_tmp as $error) {
            $warnings[$url_val][] = array('line' => trim($xpath->query('m:line', $error)->item(0)->nodeValue),
                                        'message' => trim($xpath->query('m:message', $error)->item(0)->nodeValue));
            } // foreach errors
        } // foreach urls
      } // if elements

      $out->details   = array('errors' => $errors, 'warnings' => $warnings);
      $out->fail      = false;
    } else {
      $out->fail      = true;
      $out->error_msg = '';
      $out->errors    = 0;
      $out->warnings  = 0;
      $out->details   = '';
    }

    return $out;
  } // validate_css


  // validate a single URL
  function ajax_callback_validate_url() {
    $url = trim($_POST['url']);
    //$url = 'http://109.60.71.221' . $url;

    $response_html = self::validate_html($url);
    $response_css  = self::validate_css($url);

    if ($response_html->fail == false && $response_css->fail == false) {
      global $wpdb;
      $out = new stdClass();

      $tmp = $wpdb->prepare('INSERT INTO ' . self::get_table_name() . ' (url, html_errors, html_warnings, html_details, css_errors, css_warnings, css_details, timestamp)
                             VALUES (%s, %d, %d, %s, %d, %d, %s, %s) ON DUPLICATE KEY
                             UPDATE html_errors = %d, html_warnings = %d, html_details = %s, css_errors = %d, css_warnings = %d, css_details = %s, timestamp = %s',
                             $url, $response_html->errors, $response_html->warnings, serialize($response_html->details), $response_css->errors, $response_css->warnings, serialize($response_css->details), current_time('mysql'),
                             $response_html->errors, $response_html->warnings, serialize($response_html->details), $response_css->errors, $response_css->warnings, serialize($response_css->details), current_time('mysql'));
      $wpdb->query($tmp);

      if (isset($_POST['row_id'])) {
        $out->row_id = $_POST['row_id'];
      }
      $out->html_errors   = $response_html->errors;
      $out->html_warnings = $response_html->warnings;
      $out->css_errors    = $response_css->errors;
      $out->css_warnings  = $response_css->warnings;
      $out->last          = 'a few seconds ago';
      $out->url           = $url;
      $out->fail          = false;
    } else {
      if (isset($_POST['row_id'])) {
        $out->row_id = $_POST['row_id'];
      }
      $out->html_errors   = 0;
      $out->html_warnings = 0;
      $out->css_errors    = 0;
      $out->css_warnings  = 0;
      $out->last          = 'a few seconds ago';
      $out->url           = $url;
      $out->fail          = true;
      $out->error_msg     = $response_html->error_msg;
    }

    header('Content-type: application/json');
    echo json_encode($out);
    die();
  } // ajax_callback_post_featured


  // complete options page markup
  function options_page() {
    $urls = array();
    $plugin_url = plugin_dir_url(__FILE__);

    if (!current_user_can('manage_options'))  {
      wp_die('You do not have sufficient permissions to access this page.');
    }

    echo '<div class="wrap">
          <div id="icon-validator" class="icon32"><br /></div>
          <h2>5sec Validator</h2>';

    global $post;
    $args = array('post_type' => 'any',
                  'post_status' => 'publish',
                  'posts_per_page' => -1,
                  'orderby' => 'title',
                  'order' => 'ASC');
    $the_query = new WP_Query($args);

    while ($the_query->have_posts()) {
      $the_query->the_post();
      $url = str_ireplace(home_url(), '', get_permalink($post->ID));

      $urls[] = array('url_full' => get_permalink($post->ID),
                      'url' => $url,
                      'title' => get_the_title($post->ID));
    } // while posts
    wp_reset_postdata();

    $skip = array('nav_menu', 'link_category', 'post_format');
    $taxonomies = get_taxonomies();
    foreach ($taxonomies as $taxonomy) {
      if (in_array($taxonomy, $skip)) {
        continue;
      }
      $terms = get_terms($taxonomy, array('hide_empty' => false));
      foreach ($terms as $term) {
        $url = $url = str_ireplace(home_url(), '', get_term_link($term));
        $urls[] = array('url_full' => get_term_link($term),
                        'url' => $url,
                        'title' => "{$term->taxonomy} - {$term->name}");
      }
    } // foreach taxonomy

    echo "<p>Please don't close or reload the page while validation is in progress.<br /> It takes roughly 4 seconds per page to do the validation but that greatly depends on the speed of your site and the number of errors. Results are cached so you don't have to re-validate once you leave this page.</p>";

    echo '<p><input type="button" id="validate-all" class="button-primary" value="Validate selected URLs" />';
    echo '<div id="validator-progress" style="display: none;"><span id="percentText">0%</span></div></p>';

    echo '<br />';
    echo '<table class="wp-list-table widefat" id="validator-table" cellspacing="0">';
    echo '<thead><tr>';
    echo '<th colspan="2"><input id="validate-search" value="" type="text" /> <input type="button" id="validate-search-btn" class="button-primary" value="Search" /></th>';
    echo '<th colspan="3" id="html_head">HTML</th>';
    echo '<th colspan="3" id="css_head">CSS</th>';
    echo '<th>&nbsp;</th>';
    echo '</tr>';
    echo '<tr>';
    echo '<th class="check-column"><input type="checkbox" value="1" id="chk-all" /></th>';
    echo '<th>Page URL</th>';
    echo '<th class="errors_head">Errors</th>';
    echo '<th class="warnings_head">Warnings</th>';
    echo '<th class="details_head">&nbsp;</th>';
    echo '<th class="errors_head">Errors</th>';
    echo '<th class="warnings_head">Warnings</th>';
    echo '<th class="details_head">&nbsp;</th>';
    echo '<th class="last_head">Last validation</th>';
    echo '</tr></thead>';

    $i = 1;
    foreach ($urls as $url) {
      $details = self::get_url_info($url['url_full']);

      echo '<tr id="url-' . $i . '" class="urls">';
      echo '<th scope="row" class="check-column"><input id="chk-' . $i . '" type="checkbox" class="chk-url" value="' . htmlspecialchars($url['url_full']) . '" /></th>';
      echo '<td class="url"><label for="chk-' . $i . '">' . $url['url'] . ' (' . $url['title']  . ')</label> <a href="' . $url['url_full'] . '" target="_blank"><img class="open-link" src="' . $plugin_url . 'images/open.png" title="Open URL" alt="Open URL" /></a></td>';

      if ($details) {
        echo '<td class="errors errors_html">' . $details->html_errors . '</td>';
        echo '<td class="warnings warnings_html">' . $details->html_warnings . '</td>';
        if ($details->html_errors) {
          echo '<td class="details details_html"><input type="button" class="button-secondary details-html" value="Details" href="admin-ajax.php?url='. htmlspecialchars($url['url_full']) . '&amp;action=validator_details_html" title="HTML errors details for: ' . htmlspecialchars($url['url_full']) . '" /></td>';
        } else {
          echo '<td class="details details_html"><span class="good">Valid!</span></td>';
        }
        echo '<td class="errors errors_css">' . $details->css_errors . '</td>';
        echo '<td class="warnings warnings_css">' . $details->css_warnings . '</td>';
        if ($details->css_errors || $details->css_warnings) {
          echo '<td class="details details_css"><input href="admin-ajax.php?url='. htmlspecialchars($url['url_full']) . '&amp;action=validator_details_css" title="CSS errors/warnings details for: ' . htmlspecialchars($url['url_full']) . '" type="button" class="button-secondary details-css" value="Details" /></td>';
        } else {
          echo '<td class="details details_css"><span class="good">Valid!</span></td>';
        }
        echo '<td class="last">' . date(get_option('date_format') . '<\b\r />' . get_option('time_format'), strtotime($details->timestamp)) . '</td>';
      } else {
        echo '<td class="errors errors_html">n/a</td>';
        echo '<td class="warnings warnings_html">n/a</td>';
        echo '<td class="details details_html">&nbsp;</td>';
        echo '<td class="errors errors_css">n/a</td>';
        echo '<td class="warnings warnings_css">n/a</td>';
        echo '<td class="details details_css">&nbsp;</td>';
        echo '<td class="last">never</td>';
      }

      echo '</tr>';
      $i++;
    } // while

    echo '</table>';
    echo '</div>'; // main container
  } // options_page


  // get complete info from DB
  function get_url_info($url) {
    global $wpdb;

    $tmp = $wpdb->prepare('SELECT * FROM ' . self::get_table_name() . ' WHERE url = %s LIMIT 1', $url);
    $out = $wpdb->get_row($tmp);

    return $out;
  } // get_url_info

  // clean up on deactivate
  function deactivate() {
    global $wpdb;

    $del = 'DROP TABLE IF EXISTS '. self::get_table_name();
    $wpdb->query($del);
  } // deactivate

  // plugin activation
  function activate() {
    global $wpdb;

    // prevent error in case of bad deactivation
    if ($wpdb->get_var("SHOW TABLES LIKE '" . self::get_table_name() . "'") != self::get_table_name()) {
      require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
      $sql_code = "CREATE TABLE `" . self::get_table_name() ."` (
                    `id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
                    `url` VARCHAR(512) NOT NULL,
                    `html_errors` INT(10) UNSIGNED NOT NULL,
                    `html_warnings` INT(10) UNSIGNED NOT NULL,
                    `html_details` BLOB NOT NULL,
                    `css_errors` INT(10) UNSIGNED NOT NULL,
                    `css_warnings` INT(10) UNSIGNED NOT NULL,
                    `css_details` BLOB NOT NULL,
                    `timestamp` TIMESTAMP NOT NULL,
                    PRIMARY KEY (`id`),
                    UNIQUE KEY `url` (`url`(256))
                   ) ENGINE=MYISAM DEFAULT CHARSET=utf8;";
      dbDelta($sql_code);
    }
  } // activate
} // class wf_validator


// hook everything up
add_action('init', array('wf_validator', 'init'));
register_activation_hook(__FILE__, array('wf_validator', 'activate'));
register_deactivation_hook(__FILE__, array('wf_validator', 'deactivate'));


// for sites running PHP <5.2
if (!function_exists('json_decode') || !function_exists('json_encode')) {
  require_once('json.php');

  function json_decode($var) {
    $JSON = new Services_JSON;
    return $JSON->decode($var);
  }

  function json_encode($var) {
    $JSON = new Services_JSON;
    return $JSON->encode($var);
  }
} // if not json function
?>